#!/usr/bin/python

import os
import logging
import sys
import urllib  
import urllib2
import xml.dom.minidom
from urllib2 import Request, urlopen, URLError, HTTPError 

######################################################################
# Helper script to define RewriteMap rules for Private Proxy access
# Reads RewriteMap token requests on Stdin and writes results to Stdout
# Calls to ISXC local (127.0.0.1) privateproxyaccess servlet for mapping:
#
# Inputs: 
#        encodedRequests on <stdin>
#        format is <token>:<remoteIp>
# Output: 
#        deviceIp (allow) or "NULL" (deny) string on <stdout>
#
# @author bhall
######################################################################

# DENY String: RewriteMap takes our "NULL" and maps to ""
NOT_ALLOWED = "NULL"

# Local-only tomcat HTTP connector for calling "/local/" servlet
# Allows us to work independent of apache config (enabled/disabled/user-modified ports)
LOCAL_HTTP_TOMCAT_CONNECTOR_PORT = "8011"

# The local servlet for validating token request
theservice = 'http://127.0.0.1' + ':' + LOCAL_HTTP_TOMCAT_CONNECTOR_PORT + '/local/privateproxyaccess/getMappedUrl'

# Logging to help with debugging
logger = logging.getLogger('Verify Private Proxy Access Mapper')
hdlr = logging.FileHandler('/data/logs/apache/privateproxy_error_log')
formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
hdlr.setFormatter(formatter)
logger.addHandler(hdlr)

# Change loglevel to DEBUG for verbose logging. 
logger.setLevel(logging.INFO)

logger.debug('Starting script')

def handleString(rootNode):
    stringNode = rootNode.getElementsByTagName("string")[0]
    if stringNode:
        logger.debug("Have a string node")
        child = stringNode.firstChild
        if child:
            logger.debug("Have child")
            return child.nodeValue
    else:
        logger.error("No string node found. Denying access.")
        return ""

    logger.debug("No child string")
    return ""

def handleObjectStream(response):
    rootNode = response.getElementsByTagName("object-stream")[0]
    if rootNode:
        logger.debug("Have a object-stream root node")
        return handleString(rootNode)
    else:
        return ""

# Print out hardcoded DENY string  
# Be sure to flush buffer - perl examples suggest
# buffered IO can hang apache  
def denyAccess():
    logger.debug("Deny access...")
    sys.stdout.write(NOT_ALLOWED+'\n')
    sys.stdout.flush()

# Print out response string
# Be sure to flush buffer - perl examples suggest
# buffered IO can hang apache  
def allowAccess(response):
    logger.debug("Allow access...")
    sys.stdout.write(response+'\n')
    sys.stdout.flush()

# MAIN STARTS HERE
# Script lifetime managed by httpd
while 1:
    try:
        fullRequest = sys.stdin.readline()    # Read in request
        fullRequest = fullRequest.rstrip('\n' + '\r')
 
        logger.debug("Full request is: " + fullRequest)

        # Split encodedRequest:RemoteServer
        parts = fullRequest.split(':')
        
        token = parts[0]
        remoteHost = parts[1]
        
        # Verify have token to validate
        if (token == None) or (token == ""):
            logger.error("No token in request")
            denyAccess()
            continue

        # Verify have remote host to validate
        if (remoteHost == None) or (remoteHost == ""):
            logger.error("No remoteHost")
            denyAccess()
            continue
        
        logger.debug('Calling: ' + theservice + ' with token: ' + token + ' and remotehost: ' + remoteHost)

        params = urllib.urlencode({'token': token, 'remotehost': remoteHost})
        request = urllib2.Request(theservice, params)

        # authentication is not required for 127.0.0.1 local servlets
        response = urllib2.urlopen(request)

    except HTTPError, e:
        logger.error('The server couldn\'t fulfill the request.')
        logger.error('Error code: ' + str(e.code))
        denyAccess() 
    except URLError, e:
        logger.error('We failed to reach a server.')
        logger.error('Reason: ' + str(e.reason))
        denyAccess()
    except:
        logger.error(sys.exc_info()[:2])
        denyAccess()
    else:
        responseXml = response.read()

        logger.debug('ResponseXml: ' + responseXml)
        responseDom = xml.dom.minidom.parseString(responseXml)
        logger.debug('Root tag name: ' + responseDom.documentElement.tagName)

        result = handleObjectStream(responseDom)
        if result:
                allowAccess(result)
        else:
            denyAccess()
