#!/bin/bash

#
# Contains convenience functions for various isxc shell script tasks.
#


#
# Determine the server grade.
#
# Note: Do not change or add to the output message, as it is used by callers.
#
# Returns one of ENTERPRISE, STANDARD, BASIC or VIRTUAL
#
serverGrade() {
    BASIC_NAME="BASIC"
    STANDARD_NAME="STANDARD"
    ENT_NAME="ENTERPRISE"
    VIRTUAL_NAME="VIRTUAL"

    BASIC_SKU="AP9465"
    BASIC_SKU_DELL="APC9465"
    STANDARD_SKU="AP9470"
    STANDARD_SKU_DELL="APC9470"
    ENT_SKU="AP9475"
    ENT_SKU_DELL="APC9475"

    if [ -f /data/isxc/.vm ]; then
        echo $VIRTUAL_NAME
        return
    fi

    ASSET_TAG=`getSystemId --asset-tag | gawk -F':' '{ print $2 }' | sed 's/ //'`
    if [ -n "$ASSET_TAG" ]; then
        if [[ $ASSET_TAG == $BASIC_SKU || $ASSET_TAG == $BASIC_SKU_DELL ]]; then
            echo $BASIC_NAME
            return
        elif [[ $ASSET_TAG == $STANDARD_SKU || $ASSET_TAG == $STANDARD_SKU_DELL ]]; then
            echo $STANDARD_NAME
            return
        elif [[ $ASSET_TAG == $ENT_SKU || $ASSET_TAG == $ENT_SKU_DELL ]]; then
            echo $ENT_NAME
            return
        fi
    fi

    resourceLevel
    RES_LEVEL=$?
    if [ $RES_LEVEL -eq $RES_ENTERPRISE ]; then
        echo $ENT_NAME
    elif [ $RES_LEVEL -eq $RES_STANDARD ]; then
        echo $STANDARD_NAME
    else
        echo $BASIC_NAME
    fi
}

#
# Determine the resource level of the machine, based on total RAM.
#
# Returns one of $RES_BASIC, $RES_STANDARD, $RES_ENTERPRISE 
#
resourceLevel() {
    RES_BASIC=10
    RES_STANDARD=20
    RES_ENTERPRISE=30

    TOTAL_MEMORY=`grep -i 'MemTotal' /proc/meminfo | gawk '{ print $2 }'`
    SCALED_MEM=$(($TOTAL_MEMORY / 1000))
    if [ $SCALED_MEM -gt 3800 ]; then
        return $RES_ENTERPRISE
    elif [ $SCALED_MEM -gt 1800 ]; then
        return $RES_STANDARD 
    else
        return $RES_BASIC
    fi
}

